#!/usr/local/bin/perl


# This is a sample CGI script acting as a RWhois client.  It relies on
# the standard perl5 module CGI as well as the Net::Rwhois package.
# It requires Net::Rwhois to be greater than or equal to version 0.04.

require 5.003;

use CGI;
use Net::Rwhois;

my $default_host = "root.rwhois.net";
my $default_port = 4321;

my $cgi_query = new CGI;

my $script_name = $cgi_query->url();
# have to strip out the http://hostname part
$script_name =~ s|http://[^/]+||io;

# get a query_result object based on a query.  It returns a
# QueryResult object so the script can get both responses and
# referrals.

sub query_rwhois {
  my $query_string = shift;
  my $host  = shift || $default_host;
  my $port  = shift || $default_port;

  my $connection = new Net::Rwhois::Connection( Host => $host,
                                                Port => $port );

  if (not $connection->open()) {
    print simple_error_message("Could not connect to server", "A connection could not be made to the RWhois server running at $host:$port");
    exit(0);
  }

  my $query = new Net::Rwhois::Query( Query_String => $query_string );

  my $query_result = $query->execute($connection);

  $query_result->read_results();

  $query_result;
}

# Given an array of records, format each one in a simple table.  This
# also attempts to detect normalization attributes (ID type
# attributes), and create hyperlinks back to this script.
sub simple_format_records {

  my $resp;
  my ($obj, $attr, @vals, $self_url);
  my $not_first = 0;

  if (scalar @_ > 0) {
    $resp .= "<A NAME=\"objects\">\n";
  }
  foreach $obj (@_) {

    # skip certain boring records
    next if ($obj->get_id() eq "RESERVED-1.0.0.0.0/0");

    $resp .= "<HR>\n" if ($not_first);
    $not_first++;

    $resp .= "<TABLE COLS=\"2\", BORDER=\"0\">\n";

    foreach $attr ($obj->get_attribute_names()) {
      @vals = $obj->get_attribute($attr);
      for (@vals) {
        if ($obj->get_attribute_type($attr) eq "ID") {
          $self_url = $script_name . "?" . $cgi_query->query_string();
          # massage the query string to remove the redundant host/port
          # if the URL is present.
          if ($self_url =~ /url=/io) {
            $self_url =~ s/&?host=[^&]+//io;
            $self_url =~ s/&?port=[^&]+//io;
          }
          # change the query part to the new query
          $self_url =~ s/query=[^&]+/query=$_/i;
          # escape spaces (though none should exist, really)
          # fixme: what is the canonical URL escaping routine
          $self_url =~ s/ /%20/go;
          $resp .= "<TR> <TD>$attr</TD> <TD>" .
            "<A HREF=\"$self_url\">$_</A></TD> </TR>\n";
        } else {
          $resp .= "<TR> <TD>$attr</TD> <TD>$_</TD> </TR>\n";
        }
      }
    }

    $resp .= "</TABLE>\n";
  }

  $resp;
}

# This function formats the list of referrals as simple hyperlinks.
sub simple_format_referrals {
  my $query_string = shift;
  my $resp;
  my ($referral, $url);

  if (scalar @_ > 0) {
    $resp .= "<A NAME=\"referrals\">\n<HR>\n";
  }
  foreach $referral (@_) {
    $url = $referral->get_url();

    $resp .= "Referral <A HREF=\"/cgi-bin/prwhois.cgi?url=$url&query=$query_string\">$url</A>\n";
  }

  $resp;
}

# given a list of rwhois errors, format this into an error section
sub simple_format_errors {

  my $error_string;
  my $m = "";

  return "" if (scalar @_ <= 0);

  $m .= "<HR>\n<A NAME=\"errors\">\n<B>Errors:</B>\n";

  $m .= "<UL>\n";
  foreach $error_string (@_) {
    $m .= "<LI>$error_string<\LI>\n";
  }
  $m .= "</Ul>\n";

  $m;
}

# Given an error title and a message string, format this into a valid
# http response.  This is seriously bare-bones.
sub simple_error_message {
  my $error_title = shift;
  my $message = shift;
  my $m = "";

  $m .=  $cgi_query->header();
  $m .=  $cgi_query->start_html(-title =>"Rwhois Failure: $error_title",
                               -bgcolor => "white");
  $m .=<<EOL;
<H1 align="center">$error_title</H1>
<HR>

EOL
  $m .= $message;

  $m .=  $cgi_query->end_html();

  $m;
}

sub simple_format_heading {
  my $num_objects = shift;
  my $num_referrals = shift;
  my $num_errors = shift;

  my $m = "";

  $m .= "<H1 align=\"center\">RWhois Query Results</H1>\n";

  if ($num_objects > 0) {
    $m .= "<B><A HREF=\"#objects\">Objects Returned</A>: $num_objects</B><BR>\n";
  } else {
    $m .= "<B>Objects Returned: $num_objects</B><BR>\n";
  }

  if ($num_referrals > 0) {
    $m .= "<B><A HREF=\"#referrals\">Referrals Returned</A>: $num_referrals</B><BR>\n";
  }

  if ($num_errors > 0) {
    $m .= "<B><A HREF=\"#errors\">Errors Returned</A>: $num_errors</B><BR>\n";
  }

  $m .= "<HR>\n";

  $m;
}



# Main CGI

sub main_cgi {
  my $rwhois_query = $cgi_query->param('query');

  if (! $rwhois_query){
    print simple_error_message("No Query", "There was no query specified");
    exit(0);
  }

  my $url = $cgi_query->param('url');

  if ($url) {
    my $r = new Net::Rwhois::Referral(Url => $url);

    $cgi_query->param('host', $r->get_host());
    $cgi_query->param('port', $r->get_port());
  }

  my $qr = query_rwhois($rwhois_query, $cgi_query->param('host'),
                        $cgi_query->param('port'));

  my @results = $qr->get_all_objects();

  print $cgi_query->header();
  print $cgi_query->start_html(-title => "Rwhois Results",
                               -bgcolor => "white"), "\n";

  # Output the Results
  my @results = $qr->get_all_objects();
  my @referrals = $qr->get_all_referrals();
  my @errors = $qr->get_rwhois_errors();

  print simple_format_heading(scalar @results, scalar @referrals, scalar @errors);
  print simple_format_records(@results);
  print simple_format_referrals($rwhois_query, @referrals);
  print simple_format_errors(@errors);

  print $cgi_query->end_html();
}

if (scalar @ARGV > 0) {
  $cgi_query = new CGI(*STDIN);

  main_cgi();
  print "\n";
} else{
  main_cgi();
}





